IMPLEMENTATION MODULE ObjcTool;

(*
Object Tools.

UK __DATE__ __TIME__
*)

(*IMP_SWITCHES*)

FROM RcMgr    IMPORT GRect,GPnt,RcIntersect;
FROM ObjcMgr  IMPORT Nil,Root,ObjectStates,ObjcDraw,MaxDepth,ObjcOffset,
                     ObjcChange;
FROM WindTool IMPORT GetFirstXYWH,GetNextXYWH,BeginUpdate,EndUpdate;
FROM pSTORAGE IMPORT ALLOCATE,DEALLOCATE;
FROM PORTAB   IMPORT SIGNEDWORD,UNSIGNEDWORD,ANYPOINTER,NULL;
FROM SYSTEM   IMPORT TSIZE;
CAST_IMPORT

IMPORT AES,GetObject;

PROCEDURE INCLObjectFlags(Tree : AES.TreePtr;
                          Index: AES.ObjectIndex;
                          Flags: AES.ObjectFlags);
BEGIN
#if not UNIX
  INCL(Tree^[Index].ObFlags,Flags);
#else

#endif
END INCLObjectFlags;

PROCEDURE EXCLObjectFlags(Tree : AES.TreePtr;
                          Index: AES.ObjectIndex;
                          Flags: AES.ObjectFlags);
BEGIN
#if not UNIX
  EXCL(Tree^[Index].ObFlags,Flags);
#else

#endif
END EXCLObjectFlags;

PROCEDURE INCLObjectState(Tree : AES.TreePtr;
                          Index: AES.ObjectIndex;
                          State: AES.ObjectStates);
BEGIN
#if not UNIX
  INCL(Tree^[Index].ObState,State);
#else

#endif
END INCLObjectState;

PROCEDURE EXCLObjectState(Tree : AES.TreePtr;
                          Index: AES.ObjectIndex;
                          State: AES.ObjectStates);
BEGIN
#if not UNIX
  EXCL(Tree^[Index].ObState,State);
#else

#endif
END EXCLObjectState;

PROCEDURE ObjectXYWH(    Tree : AES.TreePtr;
                         Index: AES.ObjectIndex;
                     VAR Rect : GRect);
BEGIN
  WITH Rect DO
    ObjcOffset(Tree,Index,GX,GY);
    GW:= GetObject.Width(Tree,Index);
    GH:= GetObject.Height(Tree,Index);
  END;
END ObjectXYWH;

PROCEDURE ToggleObject(Tree : AES.TreePtr;
                       Index: AES.ObjectIndex);

VAR Rect: GRect;

BEGIN
  ObjectXYWH(Tree,Root,Rect);
  ObjcChange(Tree,Index,0,Rect,
             GetObject.State(Tree,Index) / AES.ObjectState{Selected},
             TRUE);
END ToggleObject;

PROCEDURE SelectObject(Tree : AES.TreePtr;
                       Index: AES.ObjectIndex);
BEGIN
  IF NOT (Selected IN GetObject.State(Tree,Index)) THEN
    ToggleObject(Tree,Index);
  END;
END SelectObject;

PROCEDURE DeselectObject(Tree : AES.TreePtr;
                         Index: AES.ObjectIndex);
BEGIN
  IF Selected IN GetObject.State(Tree,Index) THEN
    ToggleObject(Tree,Index);
  END;
END DeselectObject;

PROCEDURE IndirectObject(Tree : AES.TreePtr;
                         Index: AES.ObjectIndex;
                         Parms: ANYPOINTER);

VAR ObExt  : POINTER TO AES.ObjectExtension;
    OldSpec: AES.ObjectSpec;

BEGIN
  OldSpec.Address:= GetObject.Spec(Tree,Index);
#if not UNIX
  ALLOCATE(ObExt,TSIZE(AES.ObjectExtension));
  IF ObExt = NIL THEN
    AES.IntOut[0]:= 0; (* error *)
    RETURN;
  ELSE
    Tree^[Index].ObSpec.UserSpec:= ObExt;
    WITH ObExt^ DO
      Spec:= OldSpec;
      Parm:= Parms;
    END;
    INCLObjectFlags(Tree,Index,AES.Indirect);
  END;
#else

#endif
END IndirectObject;

PROCEDURE DirectObject(Tree : AES.TreePtr;
                       Index: AES.ObjectIndex);

VAR OldSpec: AES.ObjectSpec;

BEGIN
  OldSpec.Address:= GetObject.Spec(Tree,Index);
#if not UNIX
  DEALLOCATE(Tree^[Index].ObSpec.Extension,TSIZE(AES.ObjectExtension));
  Tree^[Index].ObSpec:= OldSpec;
#else

#endif
  EXCLObjectFlags(Tree,Index,AES.Indirect);
END DirectObject;

PROCEDURE DrawWindowObject(Handle: SIGNEDWORD;
                           Tree  : AES.TreePtr;
                           Index : AES.ObjectIndex);

VAR Clip: GRect;
    Rect: GRect;

BEGIN
  BeginUpdate;

  GetFirstXYWH(Handle,Clip);

  WITH Clip DO
    WHILE (GW # 0) AND (GH # 0) DO
      ObjectXYWH(Tree,Index,Rect);
      IF RcIntersect(Rect,Clip) THEN
        ObjcDraw(Tree,Root,MaxDepth,Clip);
      END;
      GetNextXYWH(Handle,Clip);
    END;
  END;

  EndUpdate;
END DrawWindowObject;

PROCEDURE TreeWalk(Tree: AES.TreePtr;
                   This: AES.ObjectPtr;
                   Last: AES.ObjectPtr;
                   Work: WalkProc);

VAR Index: AES.ObjectPtr;

BEGIN
  Index:= This;

  WHILE (This # Last) AND (This # Nil) DO
    IF GetObject.Tail(Tree,This) # Index THEN
      Index:= This;
      This:= Nil;

      IF Work(Tree,Index) THEN
        This:= GetObject.Head(Tree,Index);
      END;

      IF This = Nil THEN
        This:= GetObject.Next(Tree,Index);
      END;
    ELSE
      Index:= This;
      This:= GetObject.Next(Tree,Index);
    END;
  END;
END TreeWalk;

PROCEDURE Parent(Tree: AES.TreePtr;
                 Obj : AES.ObjectPtr): AES.ObjectPtr;

VAR Parent: AES.ObjectPtr;

BEGIN
  IF Obj = Nil THEN
    RETURN Nil;
  END;

  Parent:= GetObject.Next(Tree,Obj);
  IF Parent # Nil THEN
    WHILE GetObject.Tail(Tree,Parent) # Obj DO
      Obj:= Parent;
      Parent:= GetObject.Next(Tree,Obj);
    END;
  END;
  RETURN Parent;
END Parent;

TYPE CallbackPtr = POINTER TO ObjectCallback;

PROCEDURE NewObjectCallback(Tree: AES.TreePtr;
                            Item: AES.ObjectIndex;
                            Call: ObjectCallback);

VAR Callback: CallbackPtr;

BEGIN
  IF AES.Indirect IN GetObject.Flags(Tree,Item) THEN (* just replace caller *)
#if not UNIX
    Callback:= CAST(CallbackPtr,Tree^[Item].ObSpec.Extension^.Parm);
#else

#endif
    Callback^:= Call;
  ELSE
    ALLOCATE(Callback,TSIZE(ObjectCallback));
    Callback^:= Call;
    IndirectObject(Tree,Item,Callback);
  END;
END NewObjectCallback;

PROCEDURE DisposeObjectCallback(Tree: AES.TreePtr;
                                Item: AES.ObjectIndex);

VAR Callback: CallbackPtr;

BEGIN
#if not UNIX
  Callback:= CAST(CallbackPtr,Tree^[Item].ObSpec.Extension^.Parm);
#else

#endif
  DEALLOCATE(Callback,TSIZE(ObjectCallback));
  DirectObject(Tree,Item);
END DisposeObjectCallback;

PROCEDURE ObjectCall(Tree: AES.TreePtr;
                     Item: AES.ObjectIndex);

VAR Callback: CallbackPtr;

BEGIN
#if not UNIX
  Callback:= Tree^[Item].ObSpec.Extension^.Parm;
#else

#endif
  Callback^(Tree,Item);
END ObjectCall;

PROCEDURE Indirect(Tree: AES.TreePtr;
                   Item: AES.ObjectIndex): BOOLEAN;
BEGIN
#if not UNIX
  RETURN (AES.Indirect IN GetObject.Flags(Tree,Item)) AND
         (Tree^[Item].ObSpec.Extension^.Parm # NULL);
#else
  RETURN AES.Indirect IN GetObject.Flags(Tree,Item);
#endif
END Indirect;

END ObjcTool.
